/** @file sp/spInputPlugin.h
 */

#ifndef __SPINPUTPLUGIN_H
#define __SPINPUTPLUGIN_H

#include <sp/spOutputPlugin.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @~english @defgroup spInputPluginGroup Input Plugin */

/** @addtogroup spInputPluginGroup
 *  @{ */

/** @~english
 *  @class spInputPlugin spInputPlugin.h <sp/spInputPlugin.h>
 *  @extends spOutputPlugin
 *  @brief A class to handle plugin-based input from an audio file.
@details Actually, spInputPlugin is not a class. You can handle this by using spPlugin type typedefed by `typedef struct _spPlugin spPlugin;`.
 *  @sa @ref spPluginForAudioGroup , spOutputPlugin
 */

#define SP_PLUGIN_INPUT "input 1.3"

typedef struct _spInputPluginRec spInputPluginRec;

/** @} */

#if defined(MACOS)
#pragma import on
#endif

extern spBool spIsInputPlugin(spPlugin *plugin);
/** @~english Read data from a file by using a plugin.
The difference from spReadPluginInByte() is how to specify the size of data.
 *  @param[in] plugin spPlugin pointer obtained by spOpenFilePlugin() .
 *  @param[out] data Data buffer to hold read data. The type of data depends on bits/sample of data; short for 8/16 bits, long for 24/32 bits, and double for 64 bits.
 *  @param[in] length The length of \p data . Note that this \p length is not identical to duration in points.
 *  @return Read length. This value is the same as \p length if successful.
 *  @public @memberof spInputPlugin */
extern long spReadPlugin(spPlugin *plugin, void *data, long length);
/** @~english Read data from a file by using a plugin.
The difference from spReadPlugin() is how to specify the size of data.
 *  @param[in] plugin spPlugin pointer obtained by spOpenFilePlugin() .
 *  @param[out] data Data buffer to hold read data. The type of data depends on bits/sample of data; short for 8/16 bits, long for 24/32 bits, and double for 64 bits.
 *  @param[in] data_size The size of \p data in bytes.
 *  @return Read size in bytes. This value is the same as \p data_size if successful.
 *  @public @memberof spInputPlugin */
extern long spReadPluginInByte(spPlugin *plugin, void *data, long data_size);
/** @~english Read data from a file by using a plugin. The value of the data must be basically from -1 to 1 inclusive.
The difference from spReadPluginInByte() is how to specify the size of data.
 *  @param[in] plugin spPlugin pointer obtained by spOpenFilePlugin() .
 *  @param[out] data Data buffer to hold read data. The type of data depends on bits/sample of data; short for 8/16 bits, long for 24/32 bits, and double for 64 bits.
 *  @param[in] length The length of \p data . Note that this \p length is not identical to duration in points.
 *  @param[in] weight This value is multiplied to the data after reading.
 *  @return Read length. This value is the same as \p length if successful.
 *  @public @memberof spInputPlugin */
extern long spReadPluginDoubleWeighted(spPlugin *plugin, double *data, long length, double weight);
/** @~english Read data from a file by using a plugin. The value of the data must be basically from -1 to 1 inclusive.
The difference from spReadPluginInByte() is how to specify the size of data.
 *  @param[in] plugin spPlugin pointer obtained by spOpenFilePlugin() .
 *  @param[out] data Data buffer to hold read data. The type of data depends on bits/sample of data; short for 8/16 bits, long for 24/32 bits, and double for 64 bits.
 *  @param[in] length The length of \p data . Note that this \p length is not identical to duration in points.
 *  @return Read length. This value is the same as \p length if successful.
 *  @public @memberof spInputPlugin */
extern long spReadPluginDouble(spPlugin *plugin, double *data, long length);
    
/*
 * previous frame  |-------| prev_length
 * current frame        |--------| length
 *                 |----| shift_length
 * updated data            |-----|
 *
 * if prev_length < 0, put zeros.
 */
extern long spReadPluginShift(spPlugin *plugin, long prev_length, long shift_length, void *data, long length);
extern long spReadPluginShiftDoubleWeighted(spPlugin *plugin, long prev_length, long shift_length,
                                            double *data, long length, double weight);
extern long spReadPluginShiftDouble(spPlugin *plugin, long prev_length, long shift_length,
                                    double *data, long length);

/** @~english Move the position pointer to the specified position.
 *  @param[in] plugin spPlugin pointer obtained by spOpenFilePlugin() .
 *  @param[in] pos Offset time in points.
 *  @retval SP_TRUE Success  @retval SP_FALSE Failure
 *  @public @memberof spInputPlugin */
extern spBool spSeekPlugin(spPlugin *plugin, spLong pos);

/** @~english Get the total duration of the file by using a plugin.
Note that the function call for some file formats returns always 0.
 *  @param[in] plugin spPlugin pointer obtained by spOpenFilePlugin() .
 *  @return The total duration in points of the file.
 *  @public @memberof spInputPlugin */
extern spLong spGetPluginTotalLength(spPlugin *plugin);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPINPUTPLUGIN_H */
