/*
 *        mp4info.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sp/spBaseLib.h>
#include <sp/spWave.h>
#include <sp/spMp4.h>

#include <sp/spMain.h>

#define DEBUG

#define MP4_INFO_BUF_SIZE 4096

static spBool display_song_info;
static spBool help_flag;
static int debug_level = -1;
static char o_raw_file[SP_MAX_PATHNAME] = "";
static long o_stream_id = 0;

static spOptions options;
static spOption option[] = {
    {"-song", NULL, "display song information", NULL,
         SP_TYPE_BOOLEAN, &display_song_info, SP_FALSE_STRING},
    {"-O", NULL, "output raw video to file", NULL,
         SP_TYPE_STRING_A, o_raw_file, NULL},
    {"-S", NULL, "output stream id (0 based index)", NULL,
         SP_TYPE_LONG, &o_stream_id, "0"},
    {"-debug", NULL, "debug level", NULL,
         SP_TYPE_INT, &debug_level, NULL},
    {"-help", "-h", "display this message", NULL,
         SP_TYPE_BOOLEAN, &help_flag, SP_FALSE_STRING},
};

static const char *filelabel[] = {
    "<filename...>",
};

static spBool writeRawVideo(FILE *fp, char *o_filename, spMp4Header *mp4_header, long id)
{
    spFileOffset offset;
    unsigned long sample;
    unsigned long chunk;
    unsigned long remainder;
    unsigned long sample_size;
    spMp4TrackBox *trak;
    spMp4SampleTableBox *stbl;
    FILE *o_raw_fp = NULL;
    long remain_size;
    long read_size;
    long nread, nwrite;
    char buf[MP4_INFO_BUF_SIZE];
    spBool flag = SP_TRUE;
                
    if ((trak = spFindMp4TrackBox(mp4_header, id, "vide")) != NULL) {
        if (NULL == (o_raw_fp = spOpenFile(o_filename, "wb"))) {
            spWarning("Cannot open file: %s\n", o_filename);
            return SP_FALSE;
        }

        stbl = spGetMp4TrackSampleTableBox(trak);

        for (sample = 0;; sample++) {
            offset = spConvertMp4SampleToFileOffset(stbl->stsc, 0, sample, &chunk, &remainder, &sample_size);
            spDebug(-10, "writeRawVideo", "sample = %ld, offset = %ld, chunk = %ld, remainer = %ld, sample_size = %ld\n",
                    sample, (long)offset, chunk, remainder, sample_size);
            if (offset <= 0) {
                break;
            }

            spSeekFile(fp, offset, SEEK_SET);

            remain_size = sample_size;
            while (remain_size > 0) {
                read_size = MIN(remain_size, sizeof(buf));
                if ((nread = fread(buf, 1, read_size, fp)) != read_size) {
                    spDebug(-10, "writeRawVideo", "fread error: %ld (read_size = %ld)\n", nread, read_size);
                    flag = SP_FALSE;
                    break;
                }
                if ((nwrite = fwrite(buf, 1, read_size, o_raw_fp)) != read_size) {
                    spDebug(-10, "writeRawVideo", "fwrite error: %ld (read_size = %ld)\n", nwrite, read_size);
                    flag = SP_FALSE;
                    break;
                }

                remain_size -= read_size;
            }

            if (flag == SP_FALSE) {
                break;
            }
        }

        spCloseFile(o_raw_fp);

        return flag;
    } else {
        spWarning("Cannot find track.\n");
        return SP_FALSE;
    }
}

int spMain(int argc, char *argv[])
{
    const char *filename;
    FILE *fp;
    long file_count;
    spMp4BoxSize nread;
    spMp4BoxSize total_read;
    spMp4Header mp4_header;
    spSongInfoV2 song_info;
    
    spSetHelpMessage(&help_flag, "Get MP4 file information (sample program)");
    options = spGetOptions(argc, argv, option, filelabel);
    spGetOptionsValue(argc, argv, options);

    spSetDebugLevel(debug_level);
    
    filename = spGetFile(options);
    if (filename == NULL) {
        spError(1, "Not enough files.\n");
        return 1;
    }

    file_count = 0;
    do {
        /* open file */
        if (NULL == (fp = spOpenFile(filename, "rb"))) {
            return 1;
        }
        
        printf("%s:\n", filename);
        total_read = 0;
        
        /* read information */
        if ((nread = spReadMp4Header(&mp4_header, fp)) <= 0) {
            printf("  ");
            printf("This file is not MP4 file.\n");
        } else {
            total_read += nread;

            spShowMp4HeaderInfo(&mp4_header);
            
            if (display_song_info == SP_TRUE) {
                if (spParseMp4SongInfo(mp4_header.moov->udta->meta, &song_info)) {
                    printf("\n");
                    spShowSongInfoV2(&song_info, NULL, stdout);
                    printf("\n");
                }
            }
            
            if (file_count == 0 && !spStrNone(o_raw_file)) {
                if (writeRawVideo(fp, o_raw_file, &mp4_header, o_stream_id) == SP_FALSE) {
                    spExit(1);
                    return 1;
                }
            }
            
            spFreeMp4Header(&mp4_header);
        
            ++file_count;
        }
    
        /* close file */
        spCloseFile(fp);
    } while ((filename = spGetFile(options)) != NULL);
    
    return 0;
}
