/*
 *        mp4test.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sp/spBaseLib.h>
#include <sp/spWave.h>
#include <sp/spMp4.h>
#include <sp/spMain.h>

#define DEBUG

#define MP4_TEST_BUF_SIZE 4096

static spBool help_flag;
static int debug_level = -1;

static spOptions options;
static spOption option[] = {
    {"-debug", NULL, "debug level", NULL,
         SP_TYPE_INT, &debug_level, NULL},
    {"-help", "-h", "display this message", NULL,
         SP_TYPE_BOOLEAN, &help_flag, SP_FALSE_STRING},
};

static char *filelabel[] = {
    "<input filename>",
    "<output filename>",
};

static spBool copyFile(spFileOffset size, FILE *i_fp, FILE *o_fp)
{
    spFileOffset remain_size;
    long read_size;
    long nread, nwrite;
    char buf[MP4_TEST_BUF_SIZE];
    
    remain_size = size;
    while (remain_size > 0) {
        read_size = (long)MIN(remain_size, sizeof(buf));
        if ((nread = fread(buf, 1, read_size, i_fp)) != read_size) {
            spDebug(-10, "copyFile", "fread error: %ld\n", nread);
            return SP_FALSE;
        }
        if ((nwrite = fwrite(buf, 1, read_size, o_fp)) != read_size) {
            spDebug(-10, "copyFile", "fwrite error: %ld\n", nwrite);
            return SP_FALSE;
        }

        remain_size -= read_size;
    }

    return SP_TRUE;
}

static spBool writeTest(spMp4Header *mp4_header, FILE *i_fp, FILE *o_fp)
{
    unsigned long i;
    unsigned long chunk;
    unsigned long samples_per_chunk;
    unsigned long sample;
    long sample_size;
    spMp4BoxSize nwrite;
    spFileOffset header_size;
    spFileOffset offset, o_offset;
    spMp4Box *paused_box;
    spMp4TrackBox *trak;
    spMp4SampleTableBox *stbl;

    paused_box = NULL;
    spWriteMp4Header(mp4_header, 0, &paused_box, o_fp);
    offset = spTellFile(o_fp);
    header_size = offset;

    o_offset = spGetMp4MdatContentOffset(mp4_header);
    fseek(i_fp, offset, SEEK_SET);

    spDebug(10, "writeTest", "initial frame: offset = %ld, o_offset = %ld\n", (long)offset, (long)o_offset);
    if (o_offset > offset) {
        spDebug(-10, "writeTest", "**** o_offset = %ld > offset = %ld ****\n", (long)o_offset, (long)offset);
        if (copyFile(o_offset - offset, i_fp, o_fp) == SP_FALSE) {
            return SP_FALSE;
        }
    }
    offset = o_offset;
                    
    while ((trak = spFindMp4NextChunkFromFileOffset(mp4_header, offset, &o_offset, &chunk)) != NULL) {
        spDebug(10, "writeTest", "chunk = %ld, offset = %ld, o_offset = %ld\n", chunk, (long)offset, (long)o_offset);
        if (o_offset > offset) {
            spDebug(-10, "writeTest", "**** o_offset = %ld > offset = %ld ****\n", (long)o_offset, (long)offset);
            if (copyFile(o_offset - offset, i_fp, o_fp) == SP_FALSE) {
                return SP_FALSE;
            }
        }
        offset = o_offset;
                        
        stbl = spGetMp4TrackSampleTableBox(trak);
        sample = spConvertMp4ChunkToSample(stbl->stsc, chunk, &samples_per_chunk, NULL);
        spDebug(10, "writeTest", "sample = %ld, samples_per_chunk = %ld\n", sample, samples_per_chunk);

        for (i = 0; i < samples_per_chunk; i++) {
            sample_size = spGetMp4SampleSize(stbl->stsz, sample);
            spDebug(10, "writeTest", "i = %ld, sample = %ld, sample_size = %ld\n", i, sample, sample_size);

            if (copyFile(sample_size, i_fp, o_fp) == SP_FALSE) {
                return SP_FALSE;
            }

            offset += sample_size;
            ++sample;
        }
    }

    nwrite = spRewriteMp4Header(mp4_header, offset - header_size, &paused_box, o_fp);
    spDebug(-10, "writeTest", "spRewriteMp4Header: %ld\n", (long)nwrite);
    
    return SP_TRUE;
}

int spMain(int argc, char *argv[])
{
    const char *i_filename;
    const char *o_filename;
    FILE *i_fp;
    FILE *o_fp;
    spMp4Header i_mp4_header;
    spMp4Header o_mp4_header;
    spMp4BoxSize nread;
    spMp4BoxSize total_read;
    
    spSetHelpMessage(&help_flag, "Get MP4 file information (sample program)");
    options = spGetOptions(argc, argv, option, filelabel);
    spGetOptionsValue(argc, argv, options);

    spSetDebugLevel(debug_level);
    spCheckNumFile(options);
    
    i_filename = spGetFile(options);
    o_filename = spGetFile(options);

    /* open file */
    if (NULL == (i_fp = spOpenFile(i_filename, "rb"))) {
        fprintf(stderr, "Cannot open file: %s\n", i_filename);
    } else {
        if (NULL == (o_fp = spOpenFile(o_filename, "wb"))) {
            fprintf(stderr, "Cannot open file: %s\n", o_filename);
        } else {
            printf("Input File: %s\n", i_filename);
            printf("Output File: %s\n", o_filename);
            total_read = 0;
        
            /* read information */
            if ((nread = spReadMp4Header(&i_mp4_header, i_fp)) <= 0) {
                printf("  ");
                printf("This file is not MP4 file.\n");
            } else {
                total_read += nread;

                printf("\nOriginal Information:\n");
                spShowMp4HeaderInfo(&i_mp4_header);

                spCopyMp4Header(&o_mp4_header, &i_mp4_header);
                spFreeMp4Header(&i_mp4_header);

#if 1
                printf("\n\nCopied Information:\n");
                spShowMp4HeaderInfo(&o_mp4_header);

                writeTest(&o_mp4_header, i_fp, o_fp);
#endif
                
                spFreeMp4Header(&o_mp4_header);
                
                //nwrite = (long)spWriteMp4Header(&o_mp4_header, 0, &paused_box, o_fp);
            }
            
            spCloseFile(o_fp);
        }
    
        spCloseFile(i_fp);
    }
        
    return 0;
}
